﻿// MsgLoopHandler
// (c) 2024 Felix.Froemel

// This is an include file script for Directory Opus.
// See https://www.gpsoft.com.au/endpoints/redirect.php?page=scripts for development information.

// Called by Directory Opus to initialize the include file script
function OnInitIncludeFile(initData)
{
	var versionDate = "2024-07-24";
	initData.version = "2.32";
	initData.name = "MessageLoopHandler";
	initData.desc = "Wrapper class to handle the DOpus Message Loop in event driven way. v" + initData.version + " ("+ versionDate  + ")";
	initData.copyright = "(c) 2024 Felix.Froemel";
	initData.url = "https://resource.dopus.com/t/msgloophandler-an-event-driven-wrapper-for-the-dopus-message-loop-as-include-script/51781";
	initData.min_version = "13.0";
	initData.shared = true;
}

//Wrapper class to handle the DOpus Message Loop in an event driven way.
//Call with the dialog, and optional callback for each loop step, debbugging, and listing handlers on start
function MsgLoopHandler(dlg, loopContext, dialogClosedCallback, loopStepCallback, debugLog, listHandlersOnStart) 
{
    this.Dialog = dlg;
	this.EventHandlers = [];
	this.LoopContext = loopContext || { };
	this.DialogClosedCallback = dialogClosedCallback || function(dlg) { };
    this.LoopStepCallback = loopStepCallback || function (msg) { };
    this.ListHandlersOnStart = listHandlersOnStart || false;
    this.DebugLog = debugLog || false;

	//Run the Messageloop, wait for Messageloop result
	this.Loop = function() 
	{
        if (this.ListHandlersOnStart)
            this.ListHandlers();

        var msg;
        do 
		{
            msg = this.Dialog.GetMsg();

            if (!msg.result)
			{
				if (this.DebugLog)
					this.Log("Msgloop: the dialog '" + this.Dialog.title + "' has been closed.");

				this.DialogClosedCallback(this.Dialog);
				break;
			}

            if (this.DebugLog)
			{
				var evnt =  msg.event || "no event";
				var val = String(msg.value || "no value");
				var ctrl = msg.control || "no control";
				var logMsg = "Msgloop: Event = " + evnt + "\tValue = " + val + "\tControl = " + ctrl;
                this.Log(logMsg);
			}

			//Call the actual event processing
            this.LoopStep(msg);
        }
        while (msg);
    };

	//Actual Messageloop event processing
	this.LoopStep = function (dlgMsg) 
	{
	    var controlName = dlgMsg.control;
        var event = dlgMsg.event;

        var eventHandler = this.EventHandlers[controlName];
        if (eventHandler) 
		{
        	var eventCallback = eventHandler[event];
            if (eventCallback)
				eventCallback(this.Dialog, dlgMsg, this.LoopContext);
            else //check if catchall event handler is present
			{
           		var catchAllEventHandler = eventHandler["*"];
                if (catchAllEventHandler)
					catchAllEventHandler(this.Dialog, dlgMsg, this.LoopContext);
            }
        }
		
        this.LoopStepCallback(dlgMsg);
    };

    //Add event handler callback for events
	//Checks the type of eventSenderName for either being string or array to allow adding multiple controls with the same handler as array
    //Callback Signature:
    //EventHandler(dialog, msg) or
	//EventHandler(dialog, msg, loopContext)
    //set eventtype to "*" for catching all events of this control
	/*
		Structure of EventHandlerDict is controlnames are keys to a list of mappings eventname -> eventhandler
		[
			ControlA :	[Event1, EventHandler1],
						[Event2, EventHandler2]
			ControlB : 	[Event3, EventHandler3]
			...
		]
	*/
	this.AddEventHandler = function(eventSenderName, eventType, eventCallback)
	{
		if(typeof eventSenderName === "string")
			this.AddEventHandlerInternal(eventSenderName, eventType, eventCallback);
		else if(eventSenderName instanceof Array) //adding multiple controls with the same event handler
		{
			for(var senderIndex in eventSenderName)
				this.AddEventHandlerInternal(eventSenderName[senderIndex], eventType, eventCallback);
		}
	};

	//Used for actually adding the handler
	this.AddEventHandlerInternal = function(eventSenderName, eventType, eventCallback)
	{
		if (eventSenderName in this.EventHandlers) //add to already existing entry
		{
		    var handler = this.EventHandlers[eventSenderName];
	        if (!(eventType in handler))//just add once
	            handler[eventType] = eventCallback;
			else
				this.Log("Already registered event '" + eventType + "' for sender '" + eventSenderName + "'. Skipping", true);
		}
	    else //create new element in list for eventSender and register 
		{
	    	var handler = {};
	        handler[eventType] = eventCallback;
	        this.EventHandlers[eventSenderName] = handler;
		}
	};

	//Set a timer in the dialog and add to eventhandlers
	this.SetTimer = function (timeoutMs, name, elapsedCallback) 
	{
	 	this.AddEventHandler(name, "timer", elapsedCallback);
        this.Dialog.SetTimer(timeoutMs, name);
    };

	//Watch directory for changes
	this.WatchDir = function(id, path, flags, eventCallback)
	{
		this.AddEventHandler(id, "dirchange", eventCallback);
		this.Dialog.WatchDir(id, path, flags);
	};

	//Watch tab for changes
	this.WatchTab = function(tab, events, id, eventCallback)
	{
		this.AddEventHandler(id, "tab", eventCallback);
		this.Dialog.WatchTab(tab, events, id);
	};

	//List all the handlers that are registered in the scriptlog
    this.ListHandlers = function () 
	{
        this.Log(this.DictionaryLength(this.EventHandlers) + " registered Event Handlers");
        this.PrintDict(this.EventHandlers); 
    };

	//Get element count in dictionary
	this.DictionaryLength = function(dict)
	{
		var length = 0;
		for(var keys in dict)
			length++;
		return length;
	};

	//Print all keys/dubkeys from dictionary to script log
	this.PrintDict = function (dict) 
	{
    	for (var key in dict)
			for (var subKey in dict[key])
				this.Log("\t" + key + " => " + subKey);
    };
	
	//Log events
	this.Log = function(msg, e) {
		DOpus.Output(String(msg), e || false);
	};
}